
type db

external open_db  : string -> db = "ml_sqlite3_open"
external close_db : db -> unit = "ml_sqlite3_close"

type error_code =
  | ERROR
  | INTERNAL
  | PERM
  | ABORT
  | BUSY
  | LOCKED
  | NOMEM
  | READONLY
  | INTERRUPT
  | IOERR
  | CORRUPT
  | NOTFOUND
  | FULL
  | CANTOPEN
  | PROTOCOL
  | EMPTY
  | SCHEMA
  | TOOBIG
  | CONSTRAINT
  | MISMATCH
  | MISUSE
  | NOLFS
  | AUTH
  | FORMAT
  | RANGE
  | NOTADB
exception Error of error_code * string

let init =
  Callback.register_exception "mlsqlite3_exn" (Error (ERROR, ""))

external exec  : db -> string -> unit = "ml_sqlite3_exec"
let exec_f db fmt =
  Printf.kprintf (exec db) fmt
external fetch : 
  db -> string -> 
  ?column_names:string array ref ->
  ('a -> string array -> 'a) -> 'a -> 'a = "ml_sqlite3_fetch"
external fetch_opt : 
  db -> string -> ?column_names:string array ref ->
  ('a -> string option array -> 'a) -> 'a -> 'a = "ml_sqlite_fetch3_opt"
let fetch_f db ?column_names f init fmt =
  Printf.kprintf (fun q -> fetch db q ?column_names f init) fmt

external interrupt : db -> unit = "ml_sqlite3_interrupt"

external is_complete : string -> bool = "ml_sqlite3_complete"

external busy_set : db -> (int -> [`FAIL|`RETRY]) -> unit
   = "ml_sqlite3_busy_handler"
external busy_unset : db -> unit = "ml_sqlite3_busy_handler_unset"
external busy_timeout : db -> int -> unit = "ml_sqlite3_busy_timeout"

external trace_set   : db -> (string -> unit) -> unit = "ml_sqlite3_trace"
external trace_unset : db -> unit = "ml_sqlite3_trace_unset"

external progress_handler_set : db -> int -> (unit -> unit) -> unit 
  = "ml_sqlite3_progress_handler"
external progress_handler_unset : db -> unit = "ml_sqlite3_progress_handler_unset"

external version  : unit -> string = "ml_sqlite3_version"

type sql_value
external value_blob   : sql_value -> string = "ml_sqlite3_value_blob"
external value_double : sql_value -> float  = "ml_sqlite3_value_double"
external value_int    : sql_value -> int    = "ml_sqlite3_value_int"
external value_int64  : sql_value -> int64  = "ml_sqlite3_value_int64"
external value_text   : sql_value -> string = "ml_sqlite3_value_text"
external value_type   : sql_value -> [`INTEGER|`FLOAT|`TEXT|`BLOB|`NULL] = "ml_sqlite3_value_type"

type user_fun_result = [
  | `INT of int
  | `INT64 of int64
  | `FLOAT of float
  | `TEXT of string
  | `BLOB of string
  | `VALUE of sql_value
  | `NULL ]
external _create_function : 
  db -> string -> int -> (sql_value array -> user_fun_result) -> unit
    = "ml_sqlite3_create_function"

let create_fun_N db name f =
  _create_function db name (-1) f

let create_fun_0 db name f =
  _create_function db name 0 (fun _ -> f ())

let create_fun_1 db name f =
  _create_function db name 1 (fun args -> f args.(0))

let create_fun_2 db name f =
  _create_function db name 2 (fun args -> f args.(0) args.(1))

let create_fun_3 db name f =
  _create_function db name 3 (fun args -> f args.(0) args.(1) args.(2))

external delete_function : db -> string -> unit = "ml_sqlite3_delete_function"

external last_insert_rowid : db -> int64 = "ml_sqlite3_last_insert_rowid"
external changes       : db -> int = "ml_sqlite3_changes"
external total_changes : db -> int = "ml_sqlite3_total_changes"
