val error_notice : parent:#GObj.widget -> string -> unit

type info_display
type branch_selector
type event =
  [ `CLEAR 
  | `OPEN_DB
  | `CLOSE_DB
  | `UPDATE_BEGIN
  | `UPDATE_END 
  | `NODE_SELECT of string 
  | `NODE_POPUP of string * int
  | `NODE_SWITCH_BRANCH of string]
type canvas
type keyboard_nav
type find
type t = {
  info : info_display;
  selector : branch_selector;
  canvas : canvas;
  keyboard_nav : keyboard_nav;
  find : find;
  mutable prefs : Viz_style.prefs;
  mutable db : Database.t option;
  mutable agraph : Agraph.t option;
  event_signal : event Viz_misc.Signal.t;
  mutable selected_node : string option;
  status_reporter : Status.reporter Lazy.t;
}

module Info_Display :
  sig
    val make : packing:(GObj.widget -> unit) -> info_display
    val clear_info : t -> unit
    val fetch_and_display_data : t -> string -> unit
  end

module Branch_selector :
  sig
    val make : packing:(GObj.widget -> unit) -> branch_selector * find
    val connect : t -> (Viz_types.query -> unit) -> unit
    val get_display_sub_branches : t -> bool
    val clear : t -> unit
    val populate : t -> string list -> unit
    val set_branch : t -> string -> unit
    val get_branch : t -> string option
  end

module KeyNav :
  sig
    val navigate : t -> GdkEvent.Key.t -> (string * Viz_types.c_node) option
    val select   : t -> string -> unit
    val clear    : t -> unit
  end

module Canvas :
  sig
    val make : aa:bool -> packing:(GObj.widget -> unit) -> canvas
    val zoom : t -> [< `IN | `OUT ] -> unit -> unit
    val clear : t -> unit
    val center_on : t -> string * Viz_types.c_node -> unit
    val update_graph : t -> unit
  end

module Find :
  sig
    val clear   : t -> unit
    val connect : t -> (string -> unit) -> unit
    val locate  : t -> string -> unit
    val focus_find_entry : t -> unit
  end

val make :
  aa:bool -> prefs:Viz_style.prefs -> packing:(GObj.widget -> unit) -> t
val connect_event : t -> (event -> unit) -> unit
val close : t -> unit
val finalize : t -> unit
val open_db : t -> string -> string option -> unit
val reload : t -> unit
val zoom : t -> [< `IN | `OUT ] -> unit -> unit
val display_certs : t -> string -> unit
val set_prefs : t -> Viz_style.prefs -> unit
val view_diff : t -> string -> string -> unit

val get_ancestors : t -> string -> string list
val get_toplevel : t -> GWindow.window option
